#include "season_enum.hpp"

#include <algorithm>
#include <cstdint>
#include <ostream>

namespace SeasonEnum
{

UnderlyingType to_underlying_type(Season s) noexcept
{
    return static_cast<UnderlyingType>(s);
}


std::optional<Season> from_underlying_type(UnderlyingType i) noexcept
{
    // Cast cannot be UB unless an unscoped enum with non-fixed underlying type is used.
    // Which means, we are safe. See: https://stackoverflow.com/a/18195408/6646408
    const Season s = static_cast<Season>(i);

    if (std::find(enumerators.begin(), enumerators.end(), s) == enumerators.end()) {
        return std::nullopt;
    }

    return s;
}


Season from_underlying_type(UnderlyingType i, Season fallback) noexcept
{
    const auto maybe_season = from_underlying_type(i);

    if (maybe_season) {
        return *maybe_season;
    }

    return fallback;
}


bool is_convertible_from(UnderlyingType i) noexcept
{
    return from_underlying_type(i).has_value();
}


std::string to_string(Season s)
{
    switch (s) {
        case Season::spring:
            return "spring";
        case Season::summer:
            return "summer";
        case Season::autumn:
            return "autumn";
        case Season::winter:
            return "winter";
    }
}


std::optional<Season> from_string(const std::string& str) noexcept
{
    if (str == "spring") {
        return Season::spring;
    }
    else if (str == "summer") {
        return Season::summer;
    }
    else if (str == "autumn") {
        return Season::autumn;
    }
    else if (str == "winter") {
        return Season::winter;
    }
    else {
        return std::nullopt;
    }
}


Season from_string(const std::string& str, Season fallback) noexcept
{
    const auto maybe_season = from_string(str);

    if (maybe_season) {
        return *maybe_season;
    }

    return fallback;
}


bool is_convertible_from(const std::string& str) noexcept
{
    return from_string(str).has_value();
}

}  // namespace SeasonEnum
