#include "season_enum.hpp"

#include <algorithm>
#include <cstdint>
#include <ostream>


std::optional<Season> SeasonEnum::from_underlying_type(UnderlyingType i) noexcept
{
    // Cast cannot be UB unless an unscoped enum with non-fixed underlying type is used.
    // Which means, we are safe. See: https://stackoverflow.com/a/18195408/6646408
    const Season s = static_cast<Season>(i);

    if (std::find(enumerators.begin(), enumerators.end(), s) == enumerators.end()) {
        return std::nullopt;
    }

    return s;
}


std::string SeasonEnum::to_string(Season s)
{
    switch (s) {
        case Season::spring:
            return "spring";
        case Season::summer:
            return "summer";
        case Season::autumn:
            return "autumn";
        case Season::winter:
            return "winter";
    }
}


std::optional<Season> SeasonEnum::from_string(const std::string& str) noexcept
{
    if (str == "spring") {
        return Season::spring;
    }
    else if (str == "summer") {
        return Season::summer;
    }
    else if (str == "autumn") {
        return Season::autumn;
    }
    else if (str == "winter") {
        return Season::winter;
    }
    else {
        return std::nullopt;
    }
}
